"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BitStream = void 0;
const MASKS = [
    0, 1, 3, 7, 15, 31, 63, 127, 255, 511, 1023, 2047, 4095, 8191, 16383, 32767,
    65535, 131071, 262143, 524287, 1048575, 2097151, 4194303, 8388607, 16777215,
    33554431, 67108863, 134217727, 268435455, 536870911, 1073741823, 2147483647,
    -1,
];
class BitStream {
    constructor(data) {
        this.readOffset = 0;
        this.writeOffset = 0;
        this.data = data || Buffer.alloc(0);
        this.readOffset = 0;
        this.writeOffset = 0;
    }
    getBuffer() {
        return this.data;
    }
    setBuffer(data) {
        this.data = data;
    }
    setReadOffset(offset) {
        this.readOffset = offset;
    }
    setWriteOffset(offset) {
        this.writeOffset = offset;
    }
    getReadBytesAvailable() {
        return (this.data.length * 8 - this.readOffset) >>> 3;
    }
    getWriteBytesAvailable() {
        return (this.data.length * 8 - this.writeOffset) >>> 3;
    }
    shrink() {
        this.data = this.data.slice(0, Math.ceil(this.writeOffset / 8) + 1);
    }
    ReadBits(count) {
        let result = 0;
        while (count != 0) {
            if (this.readOffset >= this.data.length * 8)
                throw new Error("End of stream reached");
            const byte = this.data[this.readOffset >> 3];
            const bit = this.readOffset & 7;
            const remainingBits = 8 - bit;
            const bitsToRead = count < remainingBits ? count : remainingBits;
            const mask = MASKS[remainingBits];
            const value = (byte & mask) >>> (remainingBits - bitsToRead);
            result |= value << (count - bitsToRead);
            count -= bitsToRead;
            this.readOffset += bitsToRead;
        }
        return result;
    }
    ReadByte() {
        return this.ReadBits(8);
    }
    ReadByteList(count) {
        const result = Buffer.alloc(count);
        for (let i = 0; i < count; i++) {
            result[i] = this.ReadByte();
        }
        return result;
    }
    ReadChar() {
        return String.fromCharCode(this.ReadByte());
    }
    ReadShort() {
        return this.ReadBits(16);
    }
    ReadInt() {
        return this.ReadBits(32);
    }
    ReadFloat() {
        const buffer = Buffer.from([this.ReadBits(32)]);
        return buffer.readFloatBE(0);
    }
    ReadString() {
        const length = this.ReadShort();
        return this.ReadByteList(length).toString("utf-8");
    }
    ReadBoolean() {
        return this.ReadBits(1) != 0;
    }
    WriteBits(value, count) {
        while (count > 0) {
            const byteOffset = this.writeOffset >>> 3;
            const bitOffset = this.writeOffset & 7;
            const bitsRemaining = 8 - bitOffset;
            let bitsToWrite;
            if (count < bitsRemaining) {
                bitsToWrite = count;
            }
            else {
                bitsToWrite = bitsRemaining;
            }
            const bits = (value & MASKS[count]) >>> (count - bitsToWrite);
            const wipeMask = MASKS[count] >>> (count - bitsToWrite);
            this.data[byteOffset] &= ~(wipeMask << (bitsRemaining - bitsToWrite));
            this.data[byteOffset] |= bits << (bitsRemaining - bitsToWrite);
            count -= bitsToWrite;
            this.writeOffset += bitsToWrite;
        }
    }
    WriteByte(value) {
        this.WriteBits(value, 8);
    }
    WriteByteList(value) {
        for (let i = 0; i < value.length; i++) {
            this.WriteByte(value[i]);
        }
    }
    WriteChar(value) {
        this.WriteByte(value.charCodeAt(0));
    }
    WriteShort(value) {
        this.WriteBits(value, 16);
    }
    WriteInt(value) {
        this.WriteBits(value, 32);
    }
    WriteFloat(value) {
        const buffer = Buffer.alloc(4);
        buffer.writeFloatBE(value, 0);
        this.WriteByteList(buffer);
    }
    WriteString(value) {
        const buffer = Buffer.from(value, "utf-8");
        this.WriteShort(buffer.length);
        this.WriteByteList(buffer);
    }
    WriteBoolean(value) {
        this.WriteBits(value ? 1 : 0, 1);
    }
}
exports.BitStream = BitStream;
