#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const os_1 = require("os");
const path_1 = __importDefault(require("path"));
const yargs_1 = __importDefault(require("yargs"));
const helpers_1 = require("yargs/helpers");
const ReplayData_1 = require("./ReplayData");
function getLatestReplay(dir) {
    if (!dir) {
        dir = path_1.default.join((0, os_1.homedir)(), 'BrawlhallaReplays');
    }
    const files = (0, fs_1.readdirSync)(dir);
    let replays = files.filter(f => f.endsWith('.replay'));
    if (replays.length < 1) {
        return false;
    }
    replays = replays.map(function (fileName) {
        return {
            name: fileName,
            time: (0, fs_1.statSync)(path_1.default.join(dir, fileName)).mtime.getTime()
        };
    }).sort(function (a, b) {
        return b.time - a.time;
    }).map(function (v) {
        return v.name;
    });
    return path_1.default.join(dir, replays[0]);
}
(0, yargs_1.default)((0, helpers_1.hideBin)(process.argv))
    .command('export <replayName|latest>', 'read a replay file', (yargs) => {
    return yargs
        .positional('replayName', {
        describe: 'the name of the replay file with or without extension. Set it to "latest" to read the latest replay file',
        demandOption: true,
        type: 'string'
    })
        .option('dir', {
        alias: 'd',
        describe: 'the directory containing the replay file, defaults to $HOME/BrawlhallaReplays',
        demandOption: false,
        type: 'string'
    })
        .option('out', {
        alias: 'o',
        describe: 'the output file',
        demandOption: false,
        type: 'string',
        default: 'replay.json'
    });
}, (argv) => {
    const { replayName, dir, out } = argv;
    let replayPath;
    if (replayName == "latest") {
        const latestReplay = getLatestReplay(dir || "");
        if (!latestReplay) {
            console.error("No replay file found");
            process.exit(1);
        }
        replayPath = latestReplay;
    }
    else if (!dir) {
        if ((0, fs_1.existsSync)(replayName)) {
            replayPath = replayName;
        }
        else {
            replayPath = path_1.default.join((0, os_1.homedir)(), 'BrawlhallaReplays', replayName);
        }
    }
    else {
        replayPath = path_1.default.resolve(dir, replayName);
    }
    if (!replayPath.endsWith('.replay')) {
        replayPath += '.replay';
    }
    if (!(0, fs_1.existsSync)(replayPath)) {
        console.error(`File ${replayPath} does not exist`);
        process.exit(1);
    }
    const replayData = (0, fs_1.readFileSync)(replayPath);
    if (!replayData) {
        console.error(`Unable to read file ${replayPath}`);
        process.exit(1);
    }
    console.log(`Reading replay file ${replayPath}`);
    const replay = ReplayData_1.ReplayData.ReadReplay(replayData);
    const jsonData = JSON.stringify(replay, null, 2);
    (0, fs_1.writeFileSync)(out, jsonData);
    console.log(`Wrote replay data to ${out}`);
    console.log(`The match was ${replay.length / 1000}s long`);
})
    .demandCommand(1)
    .help()
    .parse();
